-- Example 2 - works only for the URL  /run.lua/example2/history/Home_Page

module("example2", package.seeall)

model = require"fake_wiki_model"

function show_history(params)
    local page = model.get_page(params[1])
    local history = page:get_history(params[2], params[3], params[4])
    cgilua.htmlheader()
    cgilua.put("<H1>"..page.title.." History</H1>")
    cgilua.put("<p>Example 2 - Mapping Lua patterns to a function</p>\n")
    cgilua.put("<p>Using <code>cgilua.put</code> for content generation.</p>\n")
    cgilua.put("<p>(Links are not mapped on this page intentionally, see the Lua Gem)</p>")
    cgilua.put("<UL>")
    for i, p in ipairs(history) do
        cgilua.put(string.format(" <LI><A HREF=''>v. %s (%s)</A></LI>",
                        p.version, p.time_stamp)
        )
    end
    cgilua.put("</UL>")
end

-- Function stubs
function show_page(params)
    cgilua.htmlheader()
    cgilua.put("<H1>"..params.page_name.." (version "..params.version..")</H1>")
    cgilua.put("<p>Example 2 - Mapping Lua patterns to a function</p>\n")
    cgilua.put("<p>Using <code>cgilua.put</code> for content generation.</p>\n")
end

function show_diff(...)
    cgilua.htmlheader()
    cgilua.put("<H1>Diff Page</H1>")
end

URLs = {
   {"/show/([^%/]*)/?(%d*)", show_page},
   {"/history", show_history},
   {"/history/([^%/]*)", show_history},
   {"/history/([^%/]*)/(%d*)/?", show_history},
   {"/history/([^%/]*)/(%d*)/(%d*)/?", show_history},
   {"/history/([^%/]*)/(%d*)/(%d*)/(%d*)/?", show_history},
   {"/diff/(%d*)/(%d*)", show_diff},
}

-- Matches a regular expression pattern into an URL
function match(url, pattern)
    local match = {string.match(url, pattern .. "$")}
    return next(match) and match
end

-- Maps the correct function from an URL
function map(url) 
    for i, v in ipairs(URLs) do
        local pattern, f = unpack(v)
        local params = match(url, pattern)
        if params then 
            return f, params 
        end
    end
end

f, args = map(cgilua.script_vpath)
if f then
    return f(args)
end