-- Example 3 - works only for the URL /run.lua/example3/Home_Page.history

module("example3", package.seeall)

local model = require"fake_wiki_model" 

function show_history(page, page_name, params)
    local history = page:get_history(params.year, params.month, params.date)
    cgilua.htmlheader()
    cgilua.put("<H1>"..page.title.." History</H1>")
    cgilua.put("<p>Example 3 - Delegating action selection to objects</p>\n")
    cgilua.put("<p>Using <code>cgilua.put</code> for content generation.</p>\n")    cgilua.put("<UL>")
    for i, p in ipairs(history) do
        cgilua.put(string.format(" <LI><A HREF=%s>v. %s (%s)</A></LI>",
                        makeurl(page_name, "show", {version = p.version}),
                        p.version, p.time_stamp)
        )
    end
    cgilua.put("</UL>")
end

-- Function stubs
function show_page(page, page_name, params)
    cgilua.htmlheader()
    cgilua.put("<H1>"..page.title.." (version "..params.version..")</H1>")
    cgilua.put("<p>Example 3 - Delegating action selection to objects</p>\n")
    cgilua.put("<p>Using <code>cgilua.put</code> for content generation.</p>\n")
end

function show_diff(...)
    cgilua.htmlheader()
    cgilua.put("<H1>Diff Page</H1>")
end

function load_action_function(action_name)
    -- if action contains a dot, assume it's defined in an external module
    -- if it doesn't, assume it refers to a global function
    if string.find(action_name, "%.") then
    	local mask = "([%w_]*)%.([%w_]*)"
    	module_name, function_name = string.match(action_name, mask)
		local m = require(module_name)
		return m[function_name]
    else
       	return _M[action_name]
    end
end

function makeurl(page_name, action_name, params)
   local url = "/"..page_name
   if action_name and action_name ~= "show" then
      url = url.."."..action_name
   end
   if next(params) then
      url = url.."?"..cgilua.urlcode.encodetable(params)
   end
   return cgilua.urlpath.."/"..cgilua.app_name..url
end

function response()
	local mask = "/([%w_-]+)%.?([^%./]-)$"
	local page_name, action_name = string.match(cgilua.script_vpath, mask)
	if action_name == nil or action_name == "" then
	    action_name = "show"
	end
	local page = model.get_page(page_name, cgilua.QUERY.version)
	local action_function = load_action_function(page.actions[action_name])
	local content, content_type = action_function(page, page_name, cgilua.QUERY)
end
	
return response()
