-- Example 4 - works only for the URL  /run.lua/example4/history/Home_Page
module("example4", package.seeall)

model = require"fake_wiki_model"
require"htk"

function show_history(params)
    local page = model.get_page(params.page_name)
    local history = page:get_history(params.year, params.month, params.date)
    cgilua.htmlheader()
    cgilua.put(htk.H1{page.title.." History"})
    cgilua.put(htk.P{"Example 4 - Using HTK for content generation"})
    items = {}
    for i, p in ipairs(history) do
      	p.page_name = params.page_name
        local url = makeurl("show", p)
        table.insert(items, 
            htk.LI{ htk.A{href=url, "v. ", p.version, "(", p.time_stamp, ")"}}
        )
    end
    cgilua.put(htk.UL(items))
end

-- Function stubs
function show_page(params)
    local page = model.get_page(params.page_name)
    cgilua.htmlheader()
    cgilua.put(htk.H1{page.title.." (version "..params.version..")"})
    cgilua.put(htk.P{"Example 4 - Using HTK for content generation"})
end

function show_diff(...)
    cgilua.htmlheader()
    cgilua.put(htk.H1{"Diff Page"})
end

URLs = {
    {"/show/$page_name/$version", show_page, "show"},
    {"/history/$page_name/$year/$month/$date", show_history, "history"},
    {"/diff/$page_name/$version1/$version2", show_diff, "diff"},
}

-- Matches a $name pattern into an URL
function match(url, pattern) 
    local params = {}
    local captures = string.gsub(pattern, "(/$[%w_-]+)", "/?([^/]*)")
    local url_parts = {string.match(url, captures)}
    local i = 1
    for name in string.gmatch(pattern, "/$([%w_-]+)") do
        params[name] = url_parts[i]
        i = i + 1
    end
    return next(params) and params
end

-- Selects the correct function from an URL
function map(url) 
    for i, v in ipairs(URLs) do
        local pattern, f, name = unpack(v)
        local params = match(url, pattern)
        if params then 
            return f, params 
        end
    end
end

function makeurl(action_name, params)
    for i, v in ipairs(URLs) do
        local pattern, f, name = unpack(v)
        if name == action_name then
            local url = string.gsub(pattern, "$([%w_-]+)", params)
            url = cgilua.urlpath.."/"..cgilua.app_name..url
            return url
        end
    end
end

f, args = map(cgilua.script_vpath)
if f then
    return f(args)
end