-- Example 5 - works only for the URL /run.lua/example5/history/Home_Page
module("example5", package.seeall)

model = require"fake_wiki_model"

function show_history(params)
    local page = model.get_page(params.page_name)
    local history = page:get_history(params.year, params.month, params.date)
    local env = {
    	page = page,
    	history = history,
    	page_name = params.page_name,
    	cgilua = cgilua,
    	ipairs = ipairs,
    	makeurl = makeurl,
    }
    cgilua.handlelp ("example5.lp", env)
end

-- Function stubs
function show_page(params)
    local page = model.get_page(params.page_name)
    cgilua.htmlheader()
    cgilua.put("<H1>"..page.title.." (version "..params.version..")</H1>")
    cgilua.put("<p>Example 5 - Using Lua Pages for content generation</p>\n")
end

function show_diff(...)
    cgilua.htmlheader()
    cgilua.put("<H1>Diff Page</H1>")
end

URLs = {
    {"/show/$page_name/$version", show_page, "show"},
    {"/history/$page_name/$year/$month/$date", show_history, "history"},
    {"/diff/$page_name/$version1/$version2", show_diff, "diff"},
}

-- Matches a $name pattern into an URL
function match(url, pattern) 
    local params = {}
    local captures = string.gsub(pattern, "(/$[%w_-]+)", "/?([^/]*)")
    local url_parts = {string.match(url, captures)}
    local i = 1
    for name in string.gmatch(pattern, "/$([%w_-]+)") do
        params[name] = url_parts[i]
        i = i + 1
    end
    return next(params) and params
end

-- Maps the correct function from an URL
function map(url) 
    for i, v in ipairs(URLs) do
        local pattern, f, name = unpack(v)
        local params = match(url, pattern)
        if params then 
            return f, params 
        end
    end
end

function makeurl(action_name, params)
    for i, v in ipairs(URLs) do
        local pattern, f, name = unpack(v)
        if name == action_name then
            local url = string.gsub(pattern, "$([%w_-]+)", params)
            url = cgilua.urlpath.."/"..cgilua.app_name..url
            return url
        end
    end
end

f, args = map(cgilua.script_vpath)

if f then
    return f(args)
end