-- Example 6 - works only for the URL /run.lua/example6/history/Home_Page
module("example5", package.seeall)

model = require"fake_wiki_model"
require"cosmo"

show_history_template = [==[
<H1>$title History</H1>
<P>Example 6 - using Cosmo for content generation</P>
<UL>
 $list_versions[[<LI><A HREF="$url"> v. $version ($time_stamp)</A></LI>]]
</UL>
]==]

function show_history(params)
   local page = model.get_page(params.page_name)
   local history = page:get_history(params.year, params.month, params.date)
   cgilua.htmlheader()
   cgilua.put(
      (cosmo.fill(show_history_template,
         {title = page.title,
          list_versions = function()
             for i, p in ipairs(history) do
             	p.page_name = params.page_name
                p.url = makeurl("show", p)
                cosmo.yield(p)
             end
          end}
      ))
   )
end

-- Function stubs
function show_page(params)
    local page = model.get_page(params.page_name)
    cgilua.htmlheader()
    cgilua.put("<H1>"..page.title.." (version "..params.version..")</H1>")
    cgilua.put("<p>Example 6 - using Cosmo for content generation.</p>\n")
end

function show_diff(...)
    cgilua.htmlheader()
    cgilua.put("<H1>Diff Page</H1>")
end

URLs = {
    {"/show/$page_name/$version", show_page, "show"},
    {"/history/$page_name/$year/$month/$date", show_history, "history"},
    {"/diff/$page_name/$version1/$version2", show_diff, "diff"},
}

-- Matches a $name pattern into an URL
function match(url, pattern) 
    local params = {}
    local captures = string.gsub(pattern, "(/$[%w_-]+)", "/?([^/]*)")
    local url_parts = {string.match(url, captures)}
    local i = 1
    for name in string.gmatch(pattern, "/$([%w_-]+)") do
        params[name] = url_parts[i]
        i = i + 1
    end
    return next(params) and params
end

-- Maps the correct function from an URL
function map(url) 
    for i, v in ipairs(URLs) do
        local pattern, f, name = unpack(v)
        local params = match(url, pattern)
        if params then 
            return f, params 
        end
    end
end

function makeurl(action_name, params)
    for i, v in ipairs(URLs) do
        local pattern, f, name = unpack(v)
        if name == action_name then
            local url = string.gsub(pattern, "$([%w_-]+)", params)
            url = cgilua.urlpath.."/"..cgilua.app_name..url
            return url
        end
    end
end

f, args = map(cgilua.script_vpath)

if f then
    return f(args)
end